@@ -1,4 +1,5 @@
 /*
+//@line 42 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
 */
 
 /* This file implements the nsIHelperAppLauncherDialog interface.
@@ -43,7 +44,7 @@ nsUnknownContentTypeDialog.prototype = {
     // show: Open XUL dialog using window watcher.  Since the dialog is not
     //       modal, it needs to be a top level window and the way to open
     //       one of those is via that route).
-    show: function(aLauncher, aContext)  {  
+    show: function(aLauncher, aContext, aReason)  {
       this.mLauncher = aLauncher;
       this.mContext  = aContext;
       // Display the dialog using the Window Watcher interface.
@@ -87,58 +88,52 @@ nsUnknownContentTypeDialog.prototype = {
       // If the user is always downloading to the same location, the default download
       // folder is stored in preferences. If a value is found stored, use that 
       // automatically and don't ask via a dialog. 
-      const kDownloadFolderPref = "browser.download.defaultFolder";
       var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
-      try {
-        result = prefs.getComplexValue(kDownloadFolderPref, Components.interfaces.nsILocalFile);
-        result = this.validateLeafName(result, aDefaultFile, aSuggestedFileExtension);
-      }
-      catch (e) { 
-        // If we get here, it's because we have a new profile and the user has never configured download
-        // options, so "browser.download.defaultFolder" is not set yet. If the default is autodownload, 
-        // we need to discover the default save location. 
-        var autodownload = prefs.getBoolPref("browser.download.useDownloadDir");
-        if (autodownload) {
-          function getSpecialFolderKey(aFolderType) 
-          {
-            return "Home";
-          }
-          
-          function getDownloadsFolder(aFolder)
-          {
-            var fileLocator = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);
-
-            var dir = fileLocator.get(getSpecialFolderKey(aFolder), Components.interfaces.nsILocalFile);
-            
-            var bundle = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
-            bundle = bundle.createBundle("chrome://mozapps/locale/downloads/unknownContentType.properties");
-
-            var description = bundle.GetStringFromName("myDownloads");
-            if (aFolder != "Desktop")
-              dir.append(description);
-              
-            return dir;
-          }
-
-          var defaultFolder = null;
-          switch (prefs.getIntPref("browser.download.folderList")) {
-          case 0:
-            defaultFolder = getDownloadsFolder("Desktop")
-            break;
-          case 1:
-            defaultFolder = getDownloadsFolder("Downloads");
-            break;
-          case 2:
-            defaultFolder = prefs.getComplexValue("browser.download.dir", Components.interfaces.nsILocalFile);
-            break;
-          }
-          
-          // While we're here, set the pref too so that we don't keep coming back into this less efficient
-          // code block. 
-          prefs.setComplexValue("browser.download.defaultFolder", Components.interfaces.nsILocalFile, defaultFolder);
-          
-          result = this.validateLeafName(defaultFolder, aDefaultFile, aSuggestedFileExtension);
+      var autodownload = prefs.getBoolPref("browser.download.useDownloadDir");
+      if (autodownload) {
+        function getSpecialFolderKey(aFolderType) 
+        {
+          if (aFolderType == "Desktop")
+            return "Desk";
+        
+          if (aFolderType != "Downloads")
+            throw "ASSERTION FAILED: folder type should be 'Desktop' or 'Downloads'";
+        
+//@line 147 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
+          return "Home";
+//@line 150 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
         }
+        
+        function getDownloadsFolder(aFolder)
+        {
+          var fileLocator = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);
+
+          var dir = fileLocator.get(getSpecialFolderKey(aFolder), Components.interfaces.nsILocalFile);
+          
+          var bundle = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
+          bundle = bundle.createBundle("chrome://mozapps/locale/downloads/unknownContentType.properties");
+
+          var description = bundle.GetStringFromName("myDownloads");
+          if (aFolder != "Desktop")
+            dir.append(description);
+            
+          return dir;
+        }
+
+        var defaultFolder = null;
+        switch (prefs.getIntPref("browser.download.folderList")) {
+        case 0:
+          defaultFolder = getDownloadsFolder("Desktop");
+          break;
+        case 1:
+          defaultFolder = getDownloadsFolder("Downloads");
+          break;
+        case 2:
+          defaultFolder = prefs.getComplexValue("browser.download.dir", Components.interfaces.nsILocalFile);
+          break;
+        }
+        
+        result = this.validateLeafName(defaultFolder, aDefaultFile, aSuggestedFileExtension);
       }
       
       if (!result) {
@@ -168,7 +163,7 @@ nsUnknownContentTypeDialog.prototype = {
         var wildCardExtension = "*";
         if (aSuggestedFileExtension) {
           wildCardExtension += aSuggestedFileExtension;
-          picker.appendFilter(this.mLauncher.MIMEInfo.Description, wildCardExtension);
+          picker.appendFilter(this.mLauncher.MIMEInfo.description, wildCardExtension);
         }
 
         picker.appendFilters( nsIFilePicker.filterAll );
@@ -205,6 +200,13 @@ nsUnknownContentTypeDialog.prototype = {
     
     validateLeafName: function (aLocalFile, aLeafName, aFileExt)
     {
+      if (!aLocalFile || !aLocalFile.exists())
+        return null;
+
+      // Remove any leading periods, since we don't want to save hidden files
+      // automatically.
+      aLeafName = aLeafName.replace(/^\.+/, "");
+
       if (aLeafName == "")
         aLeafName = "unnamed" + (aFileExt ? "." + aFileExt : "");
       aLocalFile.append(aLeafName);
@@ -228,19 +230,25 @@ nsUnknownContentTypeDialog.prototype = {
         // logic to check for existing files, so we need to do that here.
         //
         // Note - this code is identical to that in 
-        //   browser/base/content/contentAreaUtils.js. 
+        //   toolkit/content/contentAreaUtils.js.
         // If you are updating this code, update that code too! We can't share code
         // here since this is called in a js component. 
+        var collisionCount = 0;
         while (aLocalFile.exists()) {
-          var parts = /.+-(\d+)(\..*)?$/.exec(aLocalFile.leafName);
-          if (parts) {
-            aLocalFile.leafName = aLocalFile.leafName.replace(/((\d+)\.)|((\d+)$)/,
-                                                              function (str, dot, dotNum, noDot, noDotNum, pos, s) {
-                                                                return (parseInt(str) + 1) + (dot ? "." : "");
-                                                              });
+          collisionCount++;
+          if (collisionCount == 1) {
+            // Append "(2)" before the last dot in (or at the end of) the filename
+            // special case .ext.gz etc files so we don't wind up with .tar(2).gz
+            if (aLocalFile.leafName.match(/\.[^\.]{1,3}\.(gz|bz2|Z)$/i)) {
+              aLocalFile.leafName = aLocalFile.leafName.replace(/\.[^\.]{1,3}\.(gz|bz2|Z)$/i, "(2)$&");
+            }
+            else {
+              aLocalFile.leafName = aLocalFile.leafName.replace(/(\.[^\.]*)?$/, "(2)$&");
+            }
           }
           else {
-            aLocalFile.leafName = aLocalFile.leafName.replace(/\.|$/, "-1$&");
+            // replace the last (n) in the filename with (n+1)
+            aLocalFile.leafName = aLocalFile.leafName.replace(/^(.*\()\d+\)/, "$1" + (collisionCount+1) + ")");
           }
         }
         aLocalFile.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0600);
@@ -290,6 +298,16 @@ nsUnknownContentTypeDialog.prototype = {
                                     aMaxTotalProgress ) {
         },
 
+        onProgressChange64: function( aWebProgress,
+                                      aRequest,
+                                      aCurSelfProgress,
+                                      aMaxSelfProgress,
+                                      aCurTotalProgress,
+                                      aMaxTotalProgress ) {
+        },
+
+
+
         onStateChange: function( aWebProgress, aRequest, aStateFlags, aStatus ) {
         },
 
@@ -303,7 +321,6 @@ nsUnknownContentTypeDialog.prototype = {
     // initDialog:  Fill various dialog fields with initial content.
     initDialog : function() {
       // Put file name in window title.
-      var win   = this.dialogElement( "unknownContentType" );
       var suggestedFileName = this.mLauncher.suggestedFileName;
 
       // Some URIs do not implement nsIURL, so we can't just QI.
@@ -327,7 +344,7 @@ nsUnknownContentTypeDialog.prototype = {
       var displayName = fname.replace(/ +/g, " ");
 
       this.mTitle = this.dialogElement("strings").getFormattedString("title", [displayName]);
-      win.setAttribute( "title", this.mTitle );
+      this.mDialog.document.title = this.mTitle;
 
       // Put content type, filename and location into intro.
       this.initIntro(url, fname, displayName);
@@ -343,6 +360,7 @@ nsUnknownContentTypeDialog.prototype = {
       // want users to be able to autodownload .exe files. 
       var rememberChoice = this.dialogElement("rememberChoice");
 
+//@line 428 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
       var mimeType = this.mLauncher.MIMEInfo.MIMEType;
       if (mimeType == "application/octet-stream" || 
           mimeType == "application/x-msdownload" ||
@@ -372,9 +390,12 @@ nsUnknownContentTypeDialog.prototype = {
     
     _timer: null,
     notify: function (aTimer) {
-      if (!this._blurred)
-        this.mDialog.document.documentElement.getButton('accept').disabled = false;
+      try { // The user may have already canceled the dialog.
+        if (!this._blurred)
+          this.mDialog.document.documentElement.getButton('accept').disabled = false;
+      } catch (ex) {}
       this._delayExpired = true;
+      this._timer = null; // the timer won't release us, so we have to release it
     },
     
     postShowCallback: function () {
@@ -430,7 +451,7 @@ nsUnknownContentTypeDialog.prototype = {
         var mimeInfo = this.mLauncher.MIMEInfo;
         
         // 1. Try to use the pretty description of the type, if one is available.
-        var typeString = mimeInfo.Description;
+        var typeString = mimeInfo.description;
         
         if (typeString == "") {
           // 2. If there is none, use the extension to identify the file, e.g. "ZIP file"
@@ -474,10 +495,12 @@ nsUnknownContentTypeDialog.prototype = {
         var result;
 
         // The checking is different on Windows...
+//@line 576 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
             // On other platforms, default is Ok if there is a default app.
             // Note that nsIMIMEInfo providers need to ensure that this holds true
             // on each platform.
         return this.mLauncher.MIMEInfo.hasDefaultHandler;
+//@line 581 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
     },
     
     // Set "default" application description field.
@@ -498,7 +521,9 @@ nsUnknownContentTypeDialog.prototype = {
 
     // getPath:
     getPath: function (aFile) {
+//@line 604 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
       return aFile.path;
+//@line 606 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
     },
 
     // initAppAndSaveToDiskValues:
@@ -516,7 +541,7 @@ nsUnknownContentTypeDialog.prototype = {
         this.dialogElement("open").disabled = true;
         var openHandler = this.dialogElement("openHandler");
         openHandler.disabled = true;
-        openHandler.label = "";
+        openHandler.selectedItem = null;
         modeGroup.selectedItem = this.dialogElement("save");
         return;
       }
@@ -566,7 +591,9 @@ nsUnknownContentTypeDialog.prototype = {
       }
       
       // otherHandler is always disabled on Mac
+//@line 677 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
       otherHandler.nextSibling.hidden = otherHandler.nextSibling.nextSibling.hidden = false;
+//@line 679 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
       this.updateOKButton();
     },
 
@@ -719,6 +746,7 @@ nsUnknownContentTypeDialog.prototype = {
           // for the file to be saved to to pass to |saveToDisk| - otherwise
           // we must ask the user to pick a save name.
 
+//@line 845 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
           this.mLauncher.saveToDisk(null, false);
         }
         else
@@ -747,7 +775,8 @@ nsUnknownContentTypeDialog.prototype = {
 
       // Cancel app launcher.
       try {
-        this.mLauncher.Cancel();
+        const NS_BINDING_ABORTED = 0x804b0002;
+        this.mLauncher.cancel(NS_BINDING_ABORTED);
       } catch(exception) {
       }
 
