@@ -16,6 +16,7 @@ const PREF_APP_UPDATE_LASTUPDATETIME_FMT  = "app.update.lastUpdateTime.%ID%";
 const PREF_APP_EXTENSIONS_VERSION         = "app.extensions.version";
 const PREF_GENERAL_USERAGENT_LOCALE       = "general.useragent.locale";
 const PREF_APP_UPDATE_INCOMPATIBLE_MODE   = "app.update.incompatible.mode";
+const PREF_UPDATE_NEVER_BRANCH            = "app.update.never.";
 
 const URI_UPDATE_PROMPT_DIALOG  = "chrome://mozapps/content/update/updates.xul";
 const URI_UPDATE_HISTORY_DIALOG = "chrome://mozapps/content/update/history.xul";
@@ -54,8 +55,8 @@ const STATE_FAILED          = "failed";
 // From updater/errors.h:
 const WRITE_ERROR = 7;
 
-const DOWNLOAD_CHUNK_SIZE           = 65536;
-const DOWNLOAD_BACKGROUND_INTERVAL  = 60;  // seconds
+const DOWNLOAD_CHUNK_SIZE           = 300000; // bytes
+const DOWNLOAD_BACKGROUND_INTERVAL  = 600;    // seconds
 const DOWNLOAD_FOREGROUND_INTERVAL  = 0;
 
 // Values for the PREF_APP_UPDATE_INCOMPATIBLE_MODE pref. See documentation in
@@ -83,17 +84,8 @@ var gABI        = null;
 var gConsole    = null;
 var gLogEnabled = { };
 
-/**
- * Logs a string to the error console. 
- * @param   string
- *          The string to write to the error console..
- */  
-function LOG(module, string) {
-  if (module in gLogEnabled) {
-    dump("*** " + module + ":" + string + "\n");
-    gConsole.logStringMessage(string);
-  }
-}
+// shared code for suppressing bad cert dialogs
+//@line 40 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/update/src/../../shared/src/badCertHandler.js"
 
 /**
  * Only allow built-in certs for HTTPS connections.  See bug 340198.
@@ -117,6 +109,75 @@ function checkCert(channel) {
     throw "cert issuer is not built-in";
 }
 
+/**
+ * This class implements nsIBadCertListener.  It's job is to prevent "bad cert"
+ * security dialogs from being shown to the user.  It is better to simply fail
+ * if the certificate is bad. See bug 304286.
+ */
+function BadCertHandler() {
+}
+BadCertHandler.prototype = {
+
+  // nsIBadCertListener
+  confirmUnknownIssuer: function(socketInfo, cert, certAddType) {
+    LOG("EM BadCertHandler: Unknown issuer");
+    return false;
+  },
+
+  confirmMismatchDomain: function(socketInfo, targetURL, cert) {
+    LOG("EM BadCertHandler: Mismatched domain");
+    return false;
+  },
+
+  confirmCertExpired: function(socketInfo, cert) {
+    LOG("EM BadCertHandler: Expired certificate");
+    return false;
+  },
+
+  notifyCrlNextupdate: function(socketInfo, targetURL, cert) {
+  },
+
+  // nsIChannelEventSink
+  onChannelRedirect: function(oldChannel, newChannel, flags) {
+    // make sure the certificate of the old channel checks out before we follow
+    // a redirect from it.  See bug 340198.
+    checkCert(oldChannel);
+  },
+
+  // nsIInterfaceRequestor
+  getInterface: function(iid) {
+    if (iid.equals(Components.interfaces.nsIBadCertListener) ||
+        iid.equals(Components.interfaces.nsIChannelEventSink))
+      return this;
+
+    Components.returnCode = Components.results.NS_ERROR_NO_INTERFACE;
+    return null;
+  },
+
+  // nsISupports
+  QueryInterface: function(iid) {
+    if (!iid.equals(Components.interfaces.nsIBadCertListener) &&
+        !iid.equals(Components.interfaces.nsIChannelEventSink) &&
+        !iid.equals(Components.interfaces.nsIInterfaceRequestor) &&
+        !iid.equals(Components.interfaces.nsISupports))
+      throw Components.results.NS_ERROR_NO_INTERFACE;
+    return this;
+  }
+};
+//@line 128 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+
+/**
+ * Logs a string to the error console. 
+ * @param   string
+ *          The string to write to the error console..
+ */  
+function LOG(module, string) {
+  if (module in gLogEnabled) {
+    dump("*** " + module + ":" + string + "\n");
+    gConsole.logStringMessage(string);
+  }
+}
+
 /**
  * Convert a string containing binary values to hex.
  */
@@ -492,6 +553,21 @@ function readStringFromFile(file) {
   return text;
 }
 
+// Read the update channel from defaults only.  We do this to ensure that
+// the channel is tightly coupled with the application and does not apply
+// to other instances of the application that may use the same profile.
+function getUpdateChannel() {
+  try {
+    var defaults =
+        gPref.QueryInterface(Components.interfaces.nsIPrefService).
+        getDefaultBranch(null);
+
+    return defaults.getCharPref(PREF_APP_UPDATE_CHANNEL);
+  } catch (e) {
+    return "default";  // failover when pref not found
+  }
+}
+
 /**
  * Update Patch
  * @param   patch
@@ -752,6 +828,9 @@ Update.prototype = {
     update.setAttribute("detailsURL", this.detailsURL);
     update.setAttribute("licenseURL", this.licenseURL);
     update.setAttribute("serviceURL", this.serviceURL);
+    // to properly support updates to the MOZILLA_1_8_BRANCH and trunk
+    // write out the channel.  see bug #368082 for more details
+    update.setAttribute("channel", getUpdateChannel());
     update.setAttribute("installDate", this.installDate);
     update.setAttribute("statusText", this.statusText);
     update.setAttribute("buildID", this.buildID);
@@ -1134,9 +1213,11 @@ UpdateService.prototype = {
     //
     // The outcome we follow is determined as follows:
     // 
+    // Note:  all Major updates require notification and confirmation
+    // 
     // Update Type      Mode      Incompatible    Outcome
-    // Major            0         Yes or No       Auto Install
-    // Major            1         No              Auto Install
+    // Major            0         Yes or No       Notify and Confirm
+    // Major            1         No              Notify and Confirm
     // Major            1         Yes             Notify and Confirm
     // Major            2         Yes or No       Notify and Confirm
     // Minor            0         Yes or No       Auto Install
@@ -1151,6 +1232,11 @@ UpdateService.prototype = {
     // If app.update.enabled is set to false, an update check is not performed
     // at all, and so none of the decision making above is entered into.
     //
+    if (update.type == "major") {
+      LOG("Checker", "_shouldPrompt: Prompting because it is a major update");
+      return true;
+    }
+
     update.QueryInterface(Components.interfaces.nsIPropertyBag);
     try {
       var licenseAccepted = update.getProperty("licenseAccepted") == "true";
@@ -1158,11 +1244,6 @@ UpdateService.prototype = {
     catch (e) {
       licenseAccepted = false;
     }
-    if (update.licenseURL && !licenseAccepted) {
-      LOG("Checker", "_shouldPrompt: Prompting because of an un-accepted " + 
-          "license");
-      return true;
-    }
     
     var updateEnabled = getPref("getBoolPref", PREF_APP_UPDATE_ENABLED, true);
     if (!updateEnabled) {
@@ -1185,18 +1266,45 @@ UpdateService.prototype = {
       LOG("Checker", "_shouldPrompt: Prompting if there are incompatibilities");
       return !isCompatible(update);
     case 2:
-      // Mode 2 is do not prompt only if there are no incompatibilities for 
-      // minor updates only
-      LOG("Checker", "_shouldPrompt: Prompting if major or there are " + 
-          "incompatibilities");
-      return update.type == "minor" ? !isCompatible(update) : true;
+      // Mode 2 is do not prompt only if there are no incompatibilities
+      LOG("Checker", "_shouldPrompt: Prompting if there are incompatibilities");
+      return !isCompatible(update);
     }
     // Mode 0 is do not prompt regardless of incompatibilities
     LOG("Checker", "_shouldPrompt: Not prompting the user - they choose to " +
         "ignore incompatibilities");
     return false;
   },
-  
+ 
+  /**
+   * Determine if we have all the strings we need to show the major update UI
+   */
+  haveStringsForMajorUpdate: function() {
+    var sbs = 
+        Components.classes["@mozilla.org/intl/stringbundle;1"].
+        getService(Components.interfaces.nsIStringBundleService);
+    var updateBundle = sbs.createBundle(URI_UPDATES_PROPERTIES);
+    
+    var stringNamesForMajorUpdate =
+        ["introType_minor_app", "introType_major_app_and_version",
+         "licenseContentNotFound", "updateMoreInfoContentNotFound",
+         "licenseContentDownloading", "updateMoreInfoContentDownloading",
+         "updatesfound_minor.title", "updatesfound_major.title",
+         "downloadButton_minor", "downloadButton_major", "neverButton"];
+
+    try {
+      for (var i = 0; i < stringNamesForMajorUpdate.length; ++i) {
+        if (!updateBundle.GetStringFromName(stringNamesForMajorUpdate[i]))
+          return false;
+      }
+      return true;
+    }
+    catch (ex) {
+      LOG("Checker", "haveStringsForMajorUpdate: missing a string: " + ex);
+      return false;
+    }
+  },
+ 
   /**
    * Determine which of the specified updates should be installed.
    * @param   updates
@@ -1221,9 +1329,21 @@ UpdateService.prototype = {
         minorUpdate = newestMinor = updates[i];
     }
 
-    // If there's a major update, always try and fetch that one first, 
-    // otherwise fall back to the newest minor update.
-    return majorUpdate || minorUpdate;
+    // XXX sspitzer hack for the MOZILLA_1_8_0_BRANCH
+    if (majorUpdate && !this.haveStringsForMajorUpdate()) {
+      LOG("Checker", "selectUpdate:  have a major update, but can't offer it because we are missing strings");
+      majorUpdate = null;
+    }
+
+    // IMPORTANT
+    // If there's a minor update, always try and fetch that one first, 
+    // otherwise use the newest major update.
+    // selectUpdate() only returns one update.
+    // if major were to trump minor, and we said "never" to the major
+    // we'd never get the minor update, since selectUpdate()
+    // would return the major update that the user said "never" to
+    // (shadowing the important minor update with security fixes)
+    return minorUpdate || majorUpdate;
   },
   
   /**
@@ -1246,6 +1366,23 @@ UpdateService.prototype = {
     if (!update)
       return;
     
+    // check if the user said "never" to this version
+    // this check is done here, and not in selectUpdate() so that
+    // the user can get an upgrade they said "never" to if they
+    // manually do "Check for Updates..."
+    // note, selectUpdate() only returns one update.
+    // but in selectUpdate(), minor updates trump major updates
+    // if major trumps minor, and we said "never" to the major
+    // we'd never see the minor update.
+    // 
+    // note, the never decision should only apply to major updates
+    // see bug #350636 for a scenario where this could potentially
+    // be an issue
+    var neverPrefName = PREF_UPDATE_NEVER_BRANCH + update.version;
+    var never = getPref("getBoolPref", neverPrefName, false);
+    if (never && update.type == "major")
+      return;
+
     if (this._shouldPrompt(update))
       showPromptIfNoIncompatibilities(update);
     else {
@@ -1581,65 +1718,6 @@ UpdateManager.prototype = {
   }
 };
 
-/**
- * This class implements nsIBadCertListener.  It's job is to prevent "bad cert"
- * security dialogs from being shown to the user.  It is better to simply fail
- * if the certificate is bad.  See bug 304286.
- */
-function BadCertHandler() {
-}
-BadCertHandler.prototype = {
-  /**
-   * See nsIBadCertListener
-   */
-  confirmUnknownIssuer: function(socketInfo, cert, certAddType) {
-    return false;
-  },
-
-  confirmMismatchDomain: function(socketInfo, targetURL, cert) {
-    return false;
-  },
-
-  confirmCertExpired: function(socketInfo, cert) {
-    return false;
-  },
-
-  notifyCrlNextupdate: function(socketInfo, targetURL, cert) {
-  },
-
-  /**
-   * See nsIChannelEventSink
-   */
-  onChannelRedirect: function(oldChannel, newChannel, flags) {
-    // make sure the certificate of the old channel checks out before we follow
-    // a redirect from it.  See bug 340198.
-    checkCert(oldChannel);
-  },
-
-  /**
-   * See nsIInterfaceRequestor
-   */
-  getInterface: function(iid) {
-    if (iid.equals(Components.interfaces.nsIBadCertListener) ||
-        iid.equals(Components.interfaces.nsIChannelEventSink))
-      return this;
-
-    Components.returnCode = Components.results.NS_ERROR_NO_INTERFACE;
-    return null;
-  },
-
-  /**
-   * See nsISupports
-   */
-  QueryInterface: function(iid) {
-    if (!iid.equals(Components.interfaces.nsIBadCertListener) &&
-        !iid.equals(Components.interfaces.nsIChannelEventSink) &&
-        !iid.equals(Components.interfaces.nsIInterfaceRequestor) &&
-        !iid.equals(Components.interfaces.nsISupports))
-      throw Components.results.NS_ERROR_NO_INTERFACE;
-    return this;
-  }
-};
 
 /**
  * Checker
@@ -1684,17 +1762,6 @@ Checker.prototype = {
       return null;
     }
 
-    // Read the update channel from defaults only.  We do this to ensure that
-    // the channel is tightly coupled with the application and does not apply
-    // to other instances of the application that may use the same profile.
-    function getUpdateChannel() {
-      try {
-        return defaults.getCharPref(PREF_APP_UPDATE_CHANNEL);
-      } catch (e) {
-        return "default";  // failover when pref not found
-      }
-    }
-
     url = url.replace(/%PRODUCT%/g, gApp.name);
     url = url.replace(/%VERSION%/g, gApp.version);
     url = url.replace(/%BUILD_ID%/g, gApp.appBuildID);
@@ -2486,7 +2553,7 @@ TimerManager.prototype = {
   }
 };
 
-//@line 2537 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2525 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
 /**
  * UpdatePrompt
  * An object which can prompt the user with information about updates, request
@@ -2628,7 +2695,7 @@ UpdatePrompt.prototype = {
     return this;
   }
 };
-//@line 2679 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2667 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
 
 var gModule = {
   registerSelf: function(componentManager, fileSpec, location, type) {
@@ -2680,13 +2747,13 @@ var gModule = {
                className  : "Update Checker",
                factory    : #1#(Checker)
              },
-//@line 2731 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2719 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
     prompt:  { CID        : Components.ID("{27ABA825-35B5-4018-9FDD-F99250A0E722}"),
                contractID : "@mozilla.org/updates/update-prompt;1",
                className  : "Update Prompt",
                factory    : #1#(UpdatePrompt)
              },
-//@line 2737 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2725 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
     timers:  { CID        : Components.ID("{B322A5C0-A419-484E-96BA-D7182163899F}"),
                contractID : "@mozilla.org/updates/timer-manager;1",
                className  : "Timer Manager",
@@ -2717,14 +2784,14 @@ function NSGetModule(compMgr, fileSpec) {
  *          the specified update, false otherwise.
  */
 function isCompatible(update) {
-//@line 2768 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2756 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
   var em = 
       Components.classes["@mozilla.org/extensions/manager;1"].
       getService(Components.interfaces.nsIExtensionManager);
   var items = em.getIncompatibleItemList("", update.extensionVersion,
     nsIUpdateItem.TYPE_ADDON, false, { });
   return items.length == 0;
-//@line 2777 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2765 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
 }
 
 /**
@@ -2743,7 +2810,7 @@ function showPromptIfNoIncompatibilities(update) {
     prompter.showUpdateAvailable(update);
   }
 
-//@line 2796 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2784 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
   /**
    * Determines if an addon is compatible with a particular update.
    * @param   addon
@@ -2843,6 +2910,6 @@ function showPromptIfNoIncompatibilities(update) {
     em.update([], 0, mode != 0, listener);
   }
   else
-//@line 2896 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2884 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
     showPrompt(update);
 }
