@@ -850,6 +850,9 @@ function getNormalizedLeafName(aFile, aDefaultExtension)
     return aFile;
 
 //@line 900 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/toolkit/content/contentAreaUtils.js"
+
+  // Remove leading dots
+  aFile = aFile.replace(/^\.+/, "");
       
   // Fix up the file name we're saving to to include the default extension
   var i = aFile.lastIndexOf(".");
